importFrom("stats", "aggregate", "approxfun", "coef", "coefficients",
           "complete.cases", "confint", "confint.default", "cov2cor", "density",
           "ecdf", "fitted", "formula", "getCall", "ks.test", "lm",
           "model.frame", "model.matrix", "na.fail", "nls", "optim", "pnorm",
           "ppoints", "predict", "punif", "qnorm", "qqplot", "quantile",
           "rchisq", "reformulate", "residuals","rnorm", "sigma", "simulate",
           "update", "update.default", "vcov", "weights")
importFrom("graphics", "abline", "legend", "lines", "polygon", "text")
importFrom("utils", "hasName")
importFrom("grDevices", "rgb")
importFrom("methods", "is")
importFrom("clue", "solve_LSAP")
importFrom("lme4", "fixef", "getME")
importFrom("calculus", "hessian", "jacobian")
export("asymptotics", "compare", "distfreereg", "rejection")
S3method(coef, distfreereg)
S3method(confint, distfreereg)
S3method(fitted, distfreereg)
S3method(formula, distfreereg)
S3method(plot, distfreereg)
S3method(predict, distfreereg)
S3method(print, compare)
S3method(print, distfreereg)
S3method(residuals, distfreereg)
S3method(update, distfreereg)
S3method(vcov, distfreereg)
S3method(ks.test, compare)
S3method(plot, compare)
