get_data.default <- function(m, ..., generation = 2){
  tryCatch(eval(getCall(m)[["data"]], envir = parent.frame(n = generation)),
           error = function(e) stop("Unable to get model object's data: ", e))
}
