simulate_response.formula <- function(true_mean, ..., true_method, true_method_args, true_data){
  if(all(c("x", "y") %in% names(formals(true_method)))){
    true_method_args[["x"]] <- TRUE
    true_method_args[["y"]] <- TRUE
  }
  model <- do.call(get(true_method),
                   args = combine_lists(list(formula = true_mean, data = true_data),
                                        true_method_args))
  simulate_response(true_mean = model, ...)
}
