validate_weights.default <- function(m, method_args = NULL){
  w <- method_args[["weights"]] %||% weights(m)
  if(!is.null(w)) validate_numeric(x = w, positive = TRUE,
                                   message = "Model weights failed validation: ")
}
