# This default method covers cases when test_mean is a function or NULL.
get_cov.default <- function(m, covariance, verbose){
  if(isTRUE(verbose)) message("Using supplied covariance structure...")
  return(covariance)
}
