get_fitted_values.function <- function(test_mean, ..., ftheta, theta_hat){
  fitted_values <- ftheta(theta_hat)
  if(!is.vector(fitted_values)){
    warning("fitted_values being coerced to a vector...")
    fitted_values <- as.vector(fitted_values)
  }
  return(fitted_values)
}