get_cov.glm <- function(m, covariance, verbose){
  if(isTRUE(verbose)) message("Extracting covariance structure from glm object...")
  covariance <- list(Sigma = m[["weights"]])
  return(covariance)
}
