validate_weights.glm <- function(m, method_args = NULL){
  if((is.null(method_args) && !is.null(getCall(m)[["weights"]])) || !is.null(method_args[["weights"]]))
    stop("'weights' argument for glm() not supported")
}