# Intended for package testing only.
all.equal.distfreereg <- function(target, current, tolerance = 1e-3, ...){
  epsp <- all.equal(target = target[["epsp"]],
                    current = current[["epsp"]],
                    tolerance = tolerance, ...)
  observed_stat <- all.equal(target = target[["observed_stat"]],
                             current = current[["observed_stat"]],
                             tolerance = tolerance, ...)
  p <- all.equal(target = target[["p"]],
                 current = current[["p"]],
                 tolerance = tolerance, ...)
  
  return(all(isTRUE(epsp), isTRUE(observed_stat), isTRUE(p)))
}
