# This function assumes that true_covariance has already been filled as needed.
create_err_dist_args_default <- function(err_dist_fun_formals, cov_args, reps, n,
                                         true_covariance){
  err_dist_args_default <- true_covariance[cov_args]
  for(a in c("reps", "n")){
    if(a %in% err_dist_fun_formals) err_dist_args_default[[a]] <- get(a)
  }
  
  return(err_dist_args_default)
}
