get_jacobian.function <- function(test_mean, ..., ftheta, theta_hat, control){
  J <- calc_jacobian(ftheta = ftheta, theta_hat = theta_hat,
                     jacobian_args = control[["jacobian_args"]])
  return(J)
}
