simulate_response.function <- function(true_mean, ..., theta, true_X, n){
  Y_mean <- as.vector(f2ftheta(true_mean, true_X, n)(theta))
  errors <- generate_error_matrix(..., n = n)
  output <- as.matrix(Y_mean + errors)
  rownames(output) <- seq_len(nrow(output))
  return(output)
}
